@@ -33,6 +33,7 @@
 #include "grn_normalizer.h"
 #include "grn_mrb.h"
 #include "grn_ctx_impl_mrb.h"
+#include "grn_ctx_impl_lua.h"
 #include "grn_logger.h"
 #include "grn_cache.h"
 #include "grn_expr.h"
@@ -51,9 +52,27 @@
 # include <netinet/in.h>
 #endif /* WIN32 */
 
-#define GRN_CTX_INITIALIZER(enc) \
-  { GRN_SUCCESS, 0, enc, 0, GRN_LOG_NOTICE,\
-    GRN_CTX_FIN, 0, 0, 0, 0, {0}, NULL, NULL, NULL, NULL, NULL }
+#define GRN_CTX_INITIALIZER(enc)                \
+  {                                             \
+    /* rc */ GRN_SUCCESS,                       \
+    /* flags */ 0,                              \
+    /* encoding */ enc,                         \
+    /* ntrace */ 0,                             \
+    /* errlvl */ GRN_LOG_NOTICE,                \
+    /* stat */ GRN_CTX_FIN,                     \
+    /* seqno */ 0,                              \
+    /* subno */ 0,                              \
+    /* seqno2 */ 0,                             \
+    /* errline */ 0,                            \
+    /* user_data */ {0},                        \
+    /* prev */ NULL,                            \
+    /* next */ NULL,                            \
+    /* errfile */ NULL,                         \
+    /* errfunc */ NULL,                         \
+    /* impl */ NULL,                            \
+    /* trace */ {0},                            \
+    /* errbuf */ {0}                            \
+  }
 
 #define GRN_CTX_CLOSED(ctx) ((ctx)->stat == GRN_CTX_FIN)
 
@@ -297,6 +316,7 @@ grn_ctx_impl_init(grn_ctx *ctx)
   ctx->impl->n_same_error_messages = 0;
 
   grn_ctx_impl_mrb_init(ctx);
+  grn_ctx_impl_lua_init(ctx);
 
   GRN_TEXT_INIT(&(ctx->impl->temporary_open_spaces.stack), 0);
   ctx->impl->temporary_open_spaces.current = NULL;
@@ -403,6 +423,7 @@ grn_ctx_impl_fin(grn_ctx *ctx)
     }
     GRN_OBJ_FIN(ctx, stack);
   }
+  grn_ctx_impl_lua_fin(ctx);
   grn_ctx_impl_mrb_fin(ctx);
   grn_ctx_loader_clear(ctx);
   if (ctx->impl->parser) {
