@@ -50,6 +50,10 @@ int grn_tokenize(const char *str, size_t str_len,
                  const char **tokbuf, int buf_size,
                  const char **rest);
 
+grn_obj *
+grn_inspect_key(grn_ctx *ctx, grn_obj *buffer,
+                grn_obj *table, const void *key, unsigned int key_size);
+
 #ifdef __cplusplus
 }
 #endif
