@@ -32,9 +32,8 @@ grn_normalizer_register(grn_ctx *ctx,
                         grn_proc_func *next,
                         grn_proc_func *fin)
 {
-  grn_expr_var vars[] = {
-    { NULL, 0 }
-  };
+  grn_expr_var vars[1];
+  memset(vars, 0, sizeof(vars));
   GRN_PTR_INIT(&vars[0].value, 0, GRN_ID_NIL);
 
   if (name_length < 0) {
@@ -980,7 +979,7 @@ grn_nfkc_normalize_unify_katakana_voiced_sound_mark(const unsigned char *utf8_ch
   return utf8_char;
 }
 
-grn_inline static const grn_bool
+grn_inline static grn_bool
 grn_nfkc_normalize_is_hyphen_famity(const unsigned char *utf8_char,
                                     size_t length)
 {
@@ -1032,7 +1031,7 @@ grn_nfkc_normalize_is_hyphen_famity(const unsigned char *utf8_char,
   return GRN_FALSE;
 }
 
-grn_inline static const grn_bool
+grn_inline static grn_bool
 grn_nfkc_normalize_is_prolonged_sound_mark_famity(const unsigned char *utf8_char,
                                                   size_t length)
 {
@@ -1141,16 +1140,16 @@ grn_nfkc_normalize_unify_stateless(grn_ctx *ctx,
     }
 
     if (data->options->unify_kana &&
-        char_type == GRN_CHAR_KATAKANA &&
+        GRN_CHAR_TYPE(char_type) == GRN_CHAR_KATAKANA &&
         unified_char_length == 3) {
       unifying = grn_nfkc_normalize_unify_kana(unifying, unified_kana);
       if (unifying == unified_kana) {
-        char_type = GRN_CHAR_HIRAGANA;
+        char_type = GRN_CHAR_HIRAGANA | (char_type & GRN_CHAR_BLANK);
       }
     }
 
     if (data->options->unify_kana_case) {
-      switch (char_type) {
+      switch (GRN_CHAR_TYPE(char_type)) {
       case GRN_CHAR_HIRAGANA :
         if (unified_char_length == 3) {
           unifying = grn_nfkc_normalize_unify_hiragana_case(unifying,
@@ -1169,7 +1168,7 @@ grn_nfkc_normalize_unify_stateless(grn_ctx *ctx,
     }
 
     if (data->options->unify_kana_voiced_sound_mark) {
-      switch (char_type) {
+      switch (GRN_CHAR_TYPE(char_type)) {
       case GRN_CHAR_HIRAGANA :
         if (unified_char_length == 3) {
           unifying = grn_nfkc_normalize_unify_hiragana_voiced_sound_mark(
@@ -1191,7 +1190,7 @@ grn_nfkc_normalize_unify_stateless(grn_ctx *ctx,
       if (grn_nfkc_normalize_is_hyphen_famity(unifying, unified_char_length)) {
         unifying = unified_hyphen;
         unified_char_length = sizeof(unified_hyphen);
-        char_type = GRN_CHAR_SYMBOL;
+        char_type = GRN_CHAR_SYMBOL | (char_type & GRN_CHAR_BLANK);
       }
     }
 
@@ -1200,7 +1199,7 @@ grn_nfkc_normalize_unify_stateless(grn_ctx *ctx,
                                                             unified_char_length)) {
         unifying = unified_prolonged_sound_mark;
         unified_char_length = sizeof(unified_prolonged_sound_mark);
-        char_type = GRN_CHAR_KATAKANA;
+        char_type = GRN_CHAR_KATAKANA | (char_type & GRN_CHAR_BLANK);
       }
     }
 
@@ -1210,7 +1209,7 @@ grn_nfkc_normalize_unify_stateless(grn_ctx *ctx,
                                                             unified_char_length)) {
         unifying = unified_hyphen;
         unified_char_length = sizeof(unified_hyphen);
-        char_type = GRN_CHAR_SYMBOL;
+        char_type = GRN_CHAR_SYMBOL | (char_type & GRN_CHAR_BLANK);
       }
     }
 
@@ -1219,7 +1218,7 @@ grn_nfkc_normalize_unify_stateless(grn_ctx *ctx,
                                                   unified_char_length)) {
         unifying = unified_middle_dot;
         unified_char_length = sizeof(unified_middle_dot);
-        char_type = GRN_CHAR_SYMBOL;
+        char_type = GRN_CHAR_SYMBOL | (char_type & GRN_CHAR_BLANK);
       }
     }
 
@@ -2256,6 +2255,68 @@ nfkc100_next(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
   }
   return NULL;
 }
+
+static void *
+nfkc121_open_options(grn_ctx *ctx,
+                     grn_obj *normalizer,
+                     grn_obj *raw_options,
+                     void *user_data)
+{
+  grn_nfkc_normalize_options *options;
+
+  options = GRN_MALLOC(sizeof(grn_nfkc_normalize_options));
+  if (!options) {
+    ERR(GRN_NO_MEMORY_AVAILABLE,
+        "[normalizer][nfkc121] "
+        "failed to allocate memory for options");
+    return NULL;
+  }
+
+  grn_nfkc121_normalize_options_init(ctx, options);
+
+  grn_nfkc_normalize_options_apply(ctx, options, raw_options);
+
+  return options;
+}
+
+static void
+nfkc121_close_options(grn_ctx *ctx, void *data)
+{
+  grn_nfkc_normalize_options *options = data;
+  grn_nfkc_normalize_options_fin(ctx, options);
+  GRN_FREE(options);
+}
+
+static grn_obj *
+nfkc121_next(grn_ctx *ctx, int nargs, grn_obj **args, grn_user_data *user_data)
+{
+  grn_obj *string = args[0];
+  grn_obj *table;
+  grn_nfkc_normalize_options *options;
+  grn_nfkc_normalize_options options_raw;
+
+  table = grn_string_get_table(ctx, string);
+  if (table) {
+    options = grn_table_cache_normalizer_options(ctx,
+                                                 table,
+                                                 string,
+                                                 nfkc121_open_options,
+                                                 nfkc121_close_options,
+                                                 NULL);
+    if (ctx->rc != GRN_SUCCESS) {
+      return NULL;
+    }
+  } else {
+    grn_nfkc121_normalize_options_init(ctx, &options_raw);
+    options = &options_raw;
+  }
+
+  grn_nfkc_normalize(ctx, string, options);
+  if (!table) {
+    grn_nfkc_normalize_options_fin(ctx, options);
+  }
+  return NULL;
+}
 #endif /* GRN_WITH_NFKC */
 
 grn_rc
@@ -2277,6 +2338,7 @@ grn_db_init_builtin_normalizers(grn_ctx *ctx)
 {
   const char *normalizer_nfkc51_name = "NormalizerNFKC51";
   const char *normalizer_nfkc100_name = "NormalizerNFKC100";
+  const char *normalizer_nfkc121_name = "NormalizerNFKC121";
 
   grn_normalizer_register(ctx, GRN_NORMALIZER_AUTO_NAME, -1,
                           NULL, auto_next, NULL);
@@ -2286,6 +2348,8 @@ grn_db_init_builtin_normalizers(grn_ctx *ctx)
                           NULL, nfkc51_next, NULL);
   grn_normalizer_register(ctx, normalizer_nfkc100_name, -1,
                           NULL, nfkc100_next, NULL);
+  grn_normalizer_register(ctx, normalizer_nfkc121_name, -1,
+                          NULL, nfkc121_next, NULL);
 #else /* GRN_WITH_NFKC */
   grn_normalizer_register(ctx, normalizer_nfkc51_name, -1,
                           NULL, NULL, NULL);
